import pygdbmi.gdbcontroller
import archr
import unittest

from common import build_container


class TestSync(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("entrypoint-env")
        build_container("offsetprinter64")
        build_container("offsetprinter32")
        build_container("stackprinter64")

    def parse_output(self, s):
        return {w.split(b":")[0]: int(w.split(b":")[1], 16) for w in s.splitlines()}

    def do_gdb(self, t):
        with archr.analyzers.GDBServerAnalyzer(t).fire_context(port=31337) as gbf:
            gc = pygdbmi.gdbcontroller.GdbController()
            gc.write("target remote %s:%d" % (t.ipv4_address, 31337))
            gc.write("continue")
            gc.exit()
            return gbf.process

    def do_qemu(self, t):
        with archr.analyzers.QEMUTracerAnalyzer(t).fire_context() as qbf:
            return qbf.process

    @unittest.skip("broken")
    def test_env(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-env").build().start() as t:
            reference_env = t.run_command(aslr=False).stdout.read()
            gdb_env = self.do_gdb(t).stdout.read()
            assert set(reference_env.splitlines()) == set(gdb_env.splitlines())
            qemu_env = self.do_qemu(t).stdout.read()
            assert set(reference_env.splitlines()) == set(qemu_env.splitlines())

    def check_offsetprinter(self, t):
        reference_str = t.run_command(aslr=False).stdout.read()
        reference_dct = self.parse_output(reference_str)
        assert self.parse_output(t.run_command(aslr=False).stdout.read()) == reference_dct

        gdb_str = self.do_gdb(t).stdout.read()
        assert self.parse_output(gdb_str) == reference_dct

        qemu_str = self.do_qemu(t).stdout.read()
        qemu_dct = self.parse_output(qemu_str)
        for s in [b"MAIN", b"STDOUT", b"SMALL_MALLOC", b"BIG_MALLOC", b"MMAP"]:
            assert hex(qemu_dct[s])[-3:] == hex(reference_dct[s])[-3:]
        assert qemu_dct[b"STACK"] - qemu_dct[b"ARGV"] == reference_dct[b"STACK"] - reference_dct[b"ARGV"]
        assert qemu_dct[b"STACK"] - qemu_dct[b"ENVP"] == reference_dct[b"STACK"] - reference_dct[b"ENVP"]

        # COMMENTED OUT PENDING LIBC INIT OFFSETS
        # dsb = archr.analyzers.DataScoutAnalyzer(t)
        # apb = archr.analyzers.angrProjectAnalyzer(t, dsb)
        # asb = archr.analyzers.angrStateAnalyzer(t, apb)
        # project = apb.fire(use_sim_procedures=False)
        # state = asb.fire(add_options={angr.sim_options.STRICT_PAGE_ACCESS}) # for now
        # simgr = project.factory.simulation_manager(state)
        ##assert not simgr.active[0].memory.load(0x7ffff7dd48f8, project.arch.bytes).symbolic # __libc_multiple_threads sanity check
        # simgr.run()
        # assert len(simgr.errored) == 0
        # assert len(simgr.deadended) == 1
        # assert len(sum(simgr.stashes.values(), [])) == 1
        ##assert simgr.deadended[0].posix.dumps(1) == reference_str

    @unittest.skip("broken")
    def test_offsetprinter64(self):
        # with archr.targets.DockerImageTarget('archr-test:offsetprinter').build().start() as t:
        t = archr.targets.DockerImageTarget("archr-test:offsetprinter64").build().start()
        self.check_offsetprinter(t)
        t.stop()

    @unittest.skip("broken")
    def test_offsetprinter32(self):
        # with archr.targets.DockerImageTarget('archr-test:offsetprinter').build().start() as t:
        t = archr.targets.DockerImageTarget("archr-test:offsetprinter32", target_arch="i386").build().start()
        self.check_offsetprinter(t)
        t.stop()

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_stack(self):
        import angr

        t = archr.targets.DockerImageTarget("archr-test:stackprinter64").build().start()
        reference_str = t.run_command(aslr=False).stdout.read()

        dsb = archr.analyzers.DataScoutAnalyzer(t)
        apb = archr.analyzers.angrProjectAnalyzer(t, dsb)
        asb = archr.analyzers.angrStateAnalyzer(t, apb)
        project = apb.fire(use_sim_procedures=False)
        state = asb.fire(add_options={angr.sim_options.STRICT_PAGE_ACCESS})  # for now
        simgr = project.factory.simulation_manager(state)
        simgr.run()
        assert len(simgr.errored) == 0
        assert len(simgr.deadended) == 1
        assert len(sum(simgr.stashes.values(), [])) == 1
        # assert simgr.deadended[0].posix.dumps(1) == reference_str

        t.stop()


if __name__ == "__main__":
    unittest.main()



import tempfile
import claripy
import shutil
import archr
import os
import unittest

from common import build_container


class TestangrAnalyzer(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("entrypoint-env")
        build_container("cat-flag")
        build_container("fauxware")
        build_container("syscall_test")

    def angr_checks(self, t):
        dsb = archr.analyzers.DataScoutAnalyzer(t)
        apb = archr.analyzers.angrProjectAnalyzer(t, dsb)
        asb = archr.analyzers.angrStateAnalyzer(t, apb)
        project = apb.fire()
        self.assertTrue(all(obj.binary.startswith("/tmp") for obj in project.loader.all_elf_objects[1:]))
        state = asb.fire()
        initial_stack = state.solver.eval(state.memory.load(state.regs.rsp, 200), cast_to=bytes)
        self.assertIn(b"ARCHR=YES", initial_stack)

        self.assertTrue(state.solver.eval_one(state.posix.brk == apb._mem_mapping["[heap]"]))
        self.assertTrue(
            state.solver.eval_one(
                (state.regs.sp + 0xFFF) & ~claripy.BVV(0xFFF, project.arch.bits) == apb._mem_mapping["[stack-end]"]
            )
        )

        # now screw with the memory map
        apb._mem_mapping["[stack-end]"] = 0x1337000
        state = asb.fire()
        self.assertTrue(
            state.solver.eval_one(
                (state.regs.sp + 0xFFF) & ~claripy.BVV(0xFFF, project.arch.bits) == apb._mem_mapping["[stack-end]"]
            )
        )

        # now check the filesystem resolution
        fd = state.posix.open("/etc/passwd", 0)
        stat = state.posix.fstat(fd)
        self.assertIsNotNone(stat)
        self.assertFalse(state.solver.symbolic(stat.st_size))
        self.assertNotEqual(state.solver.eval(stat.st_size), 0)

        # done
        project.loader.close()

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_env_angr(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-env").build().start() as t:
            self.angr_checks(t)

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_env_angr_local(self):
        tf = tempfile.mktemp()
        shutil.copy("/usr/bin/env", tf)
        with archr.targets.LocalTarget([tf], target_env=["ARCHR=YES"]).build().start() as t:
            self.angr_checks(t)
        os.unlink(tf)

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_angr_catflag(self):
        with archr.targets.DockerImageTarget("archr-test:cat-flag").build().start() as t:
            dsb = archr.analyzers.DataScoutAnalyzer(t)
            apb = archr.analyzers.angrProjectAnalyzer(t, dsb)
            asb = archr.analyzers.angrStateAnalyzer(t, apb)
            project = apb.fire()
            state = asb.fire()
            simgr = project.factory.simulation_manager(state)
            simgr.run()
            self.assertEqual(len(simgr.errored), 0)
            self.assertEqual(len(simgr.deadended), 1)
            self.assertEqual(simgr.one_deadended.posix.dumps(1), b"archr-flag\n")

    def _default_fauxware_checks(self, simgr):
        num_authed, num_rejected, num_bypassed = 0, 0, 0
        for s in simgr.deadended:
            if b"Go away" in s.posix.dumps(1):
                num_rejected += 1
            if b"Welcome to the admin console, trusted user!" in s.posix.dumps(1):
                num_authed += 1
                if b"SOSNEAKY" in s.posix.dumps(0):
                    num_bypassed += 1

        self.assertEqual(num_authed, 2)
        self.assertEqual(num_bypassed, 1)
        self.assertEqual(num_rejected, 1)

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_angr_fauxware(self):
        with archr.targets.DockerImageTarget("archr-test:fauxware").build().start() as t:
            dsb = archr.analyzers.DataScoutAnalyzer(t)
            apb = archr.analyzers.angrProjectAnalyzer(t, dsb)
            asb = archr.analyzers.angrStateAnalyzer(t, apb)
            project = apb.fire()
            state = asb.fire()
            simgr = project.factory.simulation_manager(state)
            simgr.run()
            self._default_fauxware_checks(simgr)

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_angr_fauxware_custom_plt_hooks(self):
        import angr  # pylint:disable=import-outside-toplevel

        original_puts = angr.SIM_PROCEDURES["libc"]["puts"]
        original_read = angr.SIM_PROCEDURES["posix"]["read"]

        class new_puts(angr.SimProcedure):
            def run(self, s):  # pylint:disable=arguments-differ
                self.state.globals["num_puts"] = self.state.globals.get("num_puts", 0) + 1
                return self.inline_call(original_puts, s).ret_expr

        class new_read(angr.SimProcedure):
            def run(self, fd, buf, _len):  # pylint:disable=arguments-differ
                self.state.globals["num_read"] = self.state.globals.get("num_read", 0) + 1
                return self.inline_call(original_read, fd, buf, _len).ret_expr

        with archr.targets.DockerImageTarget("archr-test:fauxware").build().start() as t:
            dsb = archr.analyzers.DataScoutAnalyzer(t)
            apb = archr.analyzers.angrProjectAnalyzer(t, dsb, custom_hooks={"puts": new_puts(), "read": new_read()})
            asb = archr.analyzers.angrStateAnalyzer(t, apb)
            project = apb.fire()
            state = asb.fire()
            simgr = project.factory.simulation_manager(state)
            simgr.run()

            num_authed, num_rejected, num_bypassed = 0, 0, 0
            for s in simgr.deadended:
                if b"Go away" in s.posix.dumps(1):
                    num_rejected += 1
                    self.assertEqual(s.globals["num_puts"], 2)
                    self.assertEqual(s.globals["num_read"], 5)
                if b"Welcome to the admin console, trusted user!" in s.posix.dumps(1):
                    num_authed += 1
                    if b"SOSNEAKY" in s.posix.dumps(0):
                        num_bypassed += 1
                        self.assertEqual(s.globals["num_puts"], 3)
                        self.assertEqual(s.globals["num_read"], 4)
                    else:
                        self.assertEqual(s.globals["num_puts"], 3)
                        self.assertEqual(s.globals["num_read"], 5)

            self.assertEqual(num_authed, 2)
            self.assertEqual(num_bypassed, 1)
            self.assertEqual(num_rejected, 1)

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_angr_fauxware_custom_binary_function_hooks(self):
        import angr  # pylint:disable=import-outside-toplevel

        class rejected(angr.SimProcedure):
            def run(self):  # pylint:disable=arguments-differ
                self.state.posix.stdout.write(None, b"Get outta here!")
                self.exit(1)

        class authorized(angr.SimProcedure):
            def run(self):  # pylint:disable=arguments-differ
                self.state.posix.stdout.write(None, b"Good on ya, mate! Get in 'ere, ya bloody admin.")

        hooks = {"accepted": authorized(), "rejected": rejected()}

        with archr.targets.DockerImageTarget("archr-test:fauxware").build().start() as t:
            dsb = archr.analyzers.DataScoutAnalyzer(t)
            apb = archr.analyzers.angrProjectAnalyzer(t, dsb, custom_hooks=hooks)
            asb = archr.analyzers.angrStateAnalyzer(t, apb)
            project = apb.fire()
            state = asb.fire()
            simgr = project.factory.simulation_manager(state)
            simgr.run()
            num_authed, num_rejected, num_bypassed = 0, 0, 0
            for s in simgr.deadended:
                if b"Get outta here!" in s.posix.dumps(1):
                    num_rejected += 1
                if b"Good on ya, mate! Get in 'ere, ya bloody admin." in s.posix.dumps(1):
                    num_authed += 1
                    if b"SOSNEAKY" in s.posix.dumps(0):
                        num_bypassed += 1

            self.assertEqual(num_authed, 2)
            self.assertEqual(num_bypassed, 1)
            self.assertEqual(num_rejected, 1)

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_angr_syscall_test(self):
        with archr.targets.DockerImageTarget("archr-test:syscall_test").build().start() as t:
            dsb = archr.analyzers.DataScoutAnalyzer(t)
            apb = archr.analyzers.angrProjectAnalyzer(t, dsb)
            asb = archr.analyzers.angrStateAnalyzer(t, apb)
            project = apb.fire()
            state = asb.fire()
            simgr = project.factory.simulation_manager(state)
            simgr.run()

            self.assertEqual(len(simgr.deadended), 1)
            (exit_code,) = (e.objects["exit_code"] for e in simgr.one_deadended.history.events if e.type == "terminate")
            self.assertEqual(simgr.one_deadended.posix.dumps(1), b"Hello, world!\n")
            self.assertEqual(simgr.one_deadended.solver.eval_one(exit_code), 42)

    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_angr_syscall_test_hooks(self):
        import angr  # pylint:disable=import-outside-toplevel

        original_write = angr.SIM_PROCEDURES["posix"]["write"]

        class new_puts(angr.SimProcedure):
            def run(self, code):  # pylint:disable=arguments-differ
                new_exit = self.state.solver.eval_one(code) + 27
                self.exit(new_exit)

        class new_write(angr.SimProcedure):
            def run(self, fd, buf, _):  # pylint:disable=arguments-differ
                self.state.globals["num_write"] = self.state.globals.get("num_read", 0) + 1
                return self.inline_call(original_write, fd, buf, 5).ret_expr

        syscalls = dict(exit=new_puts(), write=new_write())

        with archr.targets.DockerImageTarget("archr-test:syscall_test").build().start() as t:
            dsb = archr.analyzers.DataScoutAnalyzer(t)
            apb = archr.analyzers.angrProjectAnalyzer(t, dsb, custom_systemcalls=syscalls)
            asb = archr.analyzers.angrStateAnalyzer(t, apb)
            project = apb.fire()
            state = asb.fire()
            simgr = project.factory.simulation_manager(state)
            simgr.run()

            self.assertEqual(len(simgr.deadended), 1)
            (exit_code,) = (e.objects["exit_code"] for e in simgr.one_deadended.history.events if e.type == "terminate")
            self.assertEqual(simgr.one_deadended.posix.dumps(1), b"Hello")
            self.assertEqual(simgr.one_deadended.solver.eval_one(exit_code), 69)


if __name__ == "__main__":
    unittest.main()



import subprocess
import archr
import unittest

from common import build_container


class TestAnalyzerCore(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("crasher")

    def test_crasher(self):
        with archr.targets.DockerImageTarget("archr-test:crasher").build().start() as t:
            cb = archr.analyzers.CoreAnalyzer(t)
            r = cb.fire()
            assert b"LSB core file" in subprocess.check_output(["file", r.local_core_path])

    def test_crasher_noperms(self):
        with archr.targets.DockerImageTarget("archr-test:crasher").build().start(user="nobody") as t:
            cb = archr.analyzers.CoreAnalyzer(t)
            r = cb.fire()
            assert b"LSB core file" in subprocess.check_output(["file", r.local_core_path])


if __name__ == "__main__":
    unittest.main()



import socket
import archr
import time
import os
import unittest


class TestLocalTarget(unittest.TestCase):
    def test_local_cat(self):
        with archr.targets.LocalTarget(["/bin/cat"]).build().start() as t:
            p = t.run_command()
            p.stdin.write(b"Hello!\n")
            assert p.stdout.read(7) == b"Hello!\n"

    def test_local_true(self):
        with archr.targets.LocalTarget(["/bin/true"]).build().start() as t:
            p = t.run_command()
            p.wait()
            assert p.returncode == 0

    def test_local_false(self):
        with archr.targets.LocalTarget(["/bin/false"]).build().start() as t:
            p = t.run_command()
            p.wait()
            assert p.returncode == 1

    def test_local_crasher(self):
        with archr.targets.LocalTarget(
            [os.path.join(os.path.dirname(__file__), "dockers", "crasher", "crasher")]
        ).build().start() as t:
            p = t.run_command()
            p.wait()
            assert p.returncode == -11

    def test_local_nccat(self):
        with archr.targets.LocalTarget(
            "socat tcp-l:40001,reuseaddr exec:cat".split(), tcp_ports=[40001], ipv4_address="127.0.0.1"
        ).build().start() as t:
            t.run_command()
            assert t.tcp_ports == [40001]
            try:
                s = socket.create_connection((t.ipv4_address, 40001))
            except ConnectionRefusedError:
                time.sleep(5)
                s = socket.create_connection((t.ipv4_address, 40001))
            s.send(b"Hello\n")
            assert s.recv(6) == b"Hello\n"

    def test_local_env_context(self):
        with archr.targets.LocalTarget(["/usr/bin/env"], target_env=["ARCHR=HAHA"]).build().start() as t:
            with t.run_command() as p:
                stdout, _ = p.communicate()
            assert b"ARCHR=HAHA" in stdout.split(b"\n")


if __name__ == "__main__":
    unittest.main()



import archr
import os
import unittest

test_location = os.path.dirname(os.path.realpath(__file__))


class TestangrTracing(unittest.TestCase):
    @unittest.skipUnless(archr._angr_available, "angr required")
    def test_angr_tracing(self):
        target = archr.targets.LocalTarget(os.path.join(test_location, "../../binaries/tests/x86_64/true"))
        qtb = archr.analyzers.QEMUTracerAnalyzer(target)
        dsb = archr.analyzers.DataScoutAnalyzer(target, analyzer=qtb)
        apb = archr.analyzers.angrProjectAnalyzer(target, dsb)
        asb = archr.analyzers.angrStateAnalyzer(target, apb)

        trace = qtb.fire()
        p = apb.fire()
        s = asb.fire()
        tech = trace.tracer_technique()
        simgr = p.factory.simulation_manager(s)
        simgr.use_technique(tech)
        simgr.run()

        assert len(simgr.traced) == 1


if __name__ == "__main__":
    unittest.main()
